# -*- coding: utf-8 -*-
"""REST interface methods to be used in helpers

.. module:: yodalib.testenv.rest_int
   :platform: Unix
   :synopsis: REST interface methods
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core.masterhead import MasterHead
from hydratk.lib.network.rest.client import RESTClient
from hydratk.extensions.testenv.entities import Customer, Payer, Subscriber, Service, ServiceOperation
from hydratk.extensions.testenv.entities import Contact, ContactRole, Address, AddressRole
from simplejson import loads


class REST_INT(object):
    """Class REST_INT
    """

    _mh = None
    _ip = None
    _url = None
    _client = None

    def __init__(self):
        """Class constructor

        Called when the object is initialized 

        Args:
           none   

        """

        self._mh = MasterHead.get_head()
        ip = self._mh.cfg['Extensions']['TestEnv']['server_ip']
        port = self._mh.cfg['Extensions']['TestEnv']['server_port']
        self._url = 'http://{0}:{1}/rs/'.format(ip, port)
        self._client = RESTClient()

    def read_customer(self, id):
        """Method reads customer

        Args:
           id (int): customer id           

        Returns:
           obj: crm_entities.Customer

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'read_customer', msg),
                      self._mh.fromhere())

        params = {'id': id}
        url = self._url + 'customer'
        status, content = self._client.send_request(
            url, method='GET', params=params)

        if (status == 200):

            doc = loads(content)
            customer = Customer(doc['id'], doc['name'], doc['status'], doc['segment'],
                                doc['birth_no'], doc['reg_no'], doc['tax_no'])
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_found', 'customer', customer),
                          self._mh.fromhere())

            return customer

        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def create_customer(self, name, segment, status='active', birth_no=None, reg_no=None, tax_no=None):
        """Method creates customer

        Args:
           name (str): name
           segment (int): segment id, lov_segment.id
           status (str): status, lov_status.title, default active
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number      

        Returns:
           int: created customer id

        """

        msg = 'name:{0}, status:{1}, segment:{2}, birth_no:{3}, reg_no:{4}, tax_no:{5}'.format(
            name, status, segment, birth_no, reg_no, tax_no)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'create_customer', msg),
                      self._mh.fromhere())

        url = self._url + 'customer'
        customer = Customer(
            None, name, status, segment, birth_no, reg_no, tax_no)
        body = customer.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_created', 'customer', body),
                          self._mh.fromhere())
            return content
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def change_customer(self, id, name=None, status=None, segment=None, birth_no=None, reg_no=None, tax_no=None):
        """Method changes customer

        Args:
           id (int): customer id
           name (str): name
           status (str): status, lov_status.title
           segment (int): segment id, lov_segment.id           
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number      

        Returns: 
           bool: result

        """

        msg = 'id:{0}, name:{1}, status:{2}, segment:{3}, birth_no:{4}, reg_no:{5}, tax_no:{6}'. format(
            id, name, status, segment, birth_no, reg_no, tax_no)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'change_customer', msg),
                      self._mh.fromhere())

        url = self._url + 'customer'
        customer = Customer(
            id, name, status, segment, birth_no, reg_no, tax_no)
        body = customer.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_changed', 'customer', id),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def read_payer(self, id):
        """Method reads payer

        Args:
           id (int): payer id          

        Returns:
           obj: crm_entities.Payer

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'read_Payer', msg),
                      self._mh.fromhere())

        params = {'id': id}
        url = self._url + 'payer'
        status, content = self._client.send_request(
            url, method='GET', params=params)

        if (status == 200):

            doc = loads(content)
            payer = Payer(doc['id'], doc['name'], doc['status'], doc['billcycle'],
                          doc['customer'], doc['bank_account'])
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_found', 'payer', payer),
                          self._mh.fromhere())

            return payer

        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def create_payer(self, name, billcycle, customer, status='active', bank_account=None):
        """Method creates payer

        Args:
           name (str): name
           billcycle (int): billcycle, lov_billcycle.id
           customer (int): assigned customer id
           status (str): status, lov_status.title, default active
           bank_account (str): banking account            

        Returns:
           int: created payer id

        """

        msg = 'name:{0}, status:{1}, billcycle:{2}, bank_account:{3}, customer:{4}'.format(
            name, status, billcycle, bank_account, customer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'create_payer', msg),
                      self._mh.fromhere())

        url = self._url + 'payer'
        payer = Payer(None, name, status, billcycle, customer, bank_account)
        body = payer.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_created', 'payer', payer),
                          self._mh.fromhere())
            return content
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def change_payer(self, id, name=None, status=None, billcycle=None, bank_account=None, customer=None):
        """Method changes payer

        Args:
           id (int): payer id
           name (str): name
           status (str): status, lov_status.title
           billcycle (int): billcycle id, lov_billcycle.id
           bank_account (str): banking account 
           customer (int): assigned customer id                    

        Returns:
           bool: result

        """

        msg = 'id:{0}, name:{1}, status:{2}, billcycle:{3}, bank_account:{4}, customer:{5}'.format(
            id, name, status, billcycle, bank_account, customer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'change_payer', msg),
                      self._mh.fromhere())

        url = self._url + 'payer'
        payer = Payer(id, name, status, billcycle, customer, bank_account)
        body = payer.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_changed', 'payer', id),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def read_subscriber(self, id):
        """Method reads subscriber

        Args:
           id (int): subscriber id           

        Returns:
           obj: crm_entities.Subscriber

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'read_subscriber', msg),
                      self._mh.fromhere())

        params = {'id': id}
        url = self._url + 'subscriber'
        status, content = self._client.send_request(
            url, method='GET', params=params)

        if (status == 200):

            doc = loads(content)
            subscriber = Subscriber(doc['id'], doc['name'], doc['msisdn'], doc['status'], doc['market'],
                                    doc['tariff'], doc['customer'], doc['payer'])
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_found', 'subscriber', subscriber),
                          self._mh.fromhere())

            return subscriber

        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def create_subscriber(self, name, msisdn, market, tariff, customer, payer, status='active'):
        """Method creates subscriber

        Args:
           name (str): name
           msisdn (str): MSISDN
           market (int): market id, lov_market.id
           tariff (int): tariff id, lov_tariff.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           status (str): status, lov_status.title, default active                              

        Returns:
           int: created subscriber id

        """

        msg = 'name:{0}, msisdn:{1}, status:{2}, market:{3}, tariff:{4}, customer:{5}, payer:{6}'.format(
            name, msisdn, status, market, tariff, customer, payer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'create_subscriber', msg),
                      self._mh.fromhere())

        url = self._url + 'subscriber'
        subscriber = Subscriber(
            None, name, msisdn, status, market, tariff, customer, payer)
        body = subscriber.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_created', 'subscriber', content),
                          self._mh.fromhere())
            return content
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def change_subscriber(self, id, name=None, msisdn=None, status=None, market=None, tariff=None, customer=None, payer=None):
        """Method changes subscriber

        Args:
           id (int): subscriber id
           name (str): name
           msisdn (str): MSISDN
           status (str): status, lov_status.title           
           market (int): market id, lov_market.id
           tariff (int): tariff id, lov_tariff.id
           customer (int): assigned customer id
           payer (int): assigned payer id                     

        Returns:
           result: bool

        """

        msg = 'id:{0}, name:{1}, msisdn:{2}, status:{3}, market:{4}, tariff:{5}, customer:{6}, payer:{7}'.format(
            id, name, msisdn, status, market, tariff, customer, payer)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'change_subscriber', msg),
                      self._mh.fromhere())

        url = self._url + 'subscriber'
        subscriber = Subscriber(
            id, name, msisdn, status, market, tariff, customer, payer)
        body = subscriber.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_changed', 'subscriber', id),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def read_contact(self, id):
        """Method reads contact

        Args:
           id (int): contact id         

        Returns:
           obj: crm_entities.Contact

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'read_contact', msg),
                      self._mh.fromhere())

        params = {'id': id}
        url = self._url + 'contact'
        status, content = self._client.send_request(
            url, method='GET', params=params)

        if (status == 200):

            doc = loads(content)
            roles = []
            if ('roles' in doc):
                for role in doc['roles']['role']:
                    roles.append(ContactRole(role['id'], role['title'], role['customer'],
                                             role['payer'], role['subscriber']))

            contact = Contact(
                doc['id'], doc['name'], doc['phone'], doc['email'], roles)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_found', 'contact', contact),
                          self._mh.fromhere())

            return contact

        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def create_contact(self, name, phone=None, email=None):
        """Method creates contact

        Args:
           name (str): name
           phone (str): phone number
           email (str): email         

        Returns:
           int: created contact id

        """

        msg = 'name:{0}, phone:{1}, email:{2}'.format(name, phone, email)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'create_contact', msg),
                      self._mh.fromhere())

        url = self._url + 'contact'
        contact = Contact(None, name, phone, email)
        body = contact.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_created', 'contact', content),
                          self._mh.fromhere())
            return content
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def change_contact(self, id, name=None, phone=None, email=None):
        """Method changes contact

        Args:
           id (int): contact id
           name (str): name
           phone (str): phone number
           email (str): email         

        Returns:
           bool: result

        """

        msg = 'id:{0}, name:{1}, phone:{2}, email:{3}'.format(
            id, name, phone, email)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'change_contact', msg),
                      self._mh.fromhere())

        url = self._url + 'contact'
        contact = Contact(id, name, phone, email)
        body = contact.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_changed', 'contact', id),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def assign_contact_role(self, id, role, customer=None, payer=None, subscriber=None):
        """Method assigns contact role

        Args:
           id (int): contact id
           role (str): role title, lov_contact_role.title
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id     

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, customer:{2}, payer:{3}, subscriber:{4}'.format(
            id, role, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'assign_contact_role', msg),
                      self._mh.fromhere())

        url = self._url + 'contact/role'
        contact_role = ContactRole(id, role, customer, payer, subscriber)
        body = contact_role.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_role_assigned', 'contact'),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def revoke_contact_role(self, id, role, customer=None, payer=None, subscriber=None):
        """Method revokes contact role

        Args:
           id (int): contact id
           role (str): role title, lov_contact_role.title
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id     

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, customer:{2}, payer:{3}, subscriber:{4}'.format(
            id, role, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'revoke_contact_role', msg),
                      self._mh.fromhere())

        url = self._url + 'contact/role'
        contact_role = ContactRole(id, role, customer, payer, subscriber)
        body = contact_role.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_role_revoked', 'contact'),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def read_address(self, id):
        """Method reads address

        Args:
           id (int): address id         

        Returns:
           obj: crm_entities.Address 

        """

        msg = 'id:{0}'.format(id)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'read_address', msg),
                      self._mh.fromhere())

        params = {'id': id}
        url = self._url + 'address'
        status, content = self._client.send_request(
            url, method='GET', params=params)

        if (status == 200):

            doc = loads(content)

            roles = []
            if ('roles' in doc):
                for role in doc['roles']['role']:
                    roles.append(AddressRole(role['id'], role['title'], role['contact'],
                                             role['customer'], role['payer'], role['subscriber']))

            address = Address(
                doc['id'], doc['street'], doc['street_no'], doc['city'], doc['zip'], roles)
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_found', 'address', address),
                          self._mh.fromhere())

            return address

        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def create_address(self, street, street_no, city, zip):
        """Method creates address

        Args:
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code       

        Returns:
           int: created address id

        """

        msg = 'street:{0}, street_no:{1}, city:{2}, zip:{3}'.format(
            street, street_no, city, zip)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'create_address', msg),
                      self._mh.fromhere())

        url = self._url + 'address'
        address = Address(None, street, street_no, city, zip)
        body = address.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_created', 'address', content),
                          self._mh.fromhere())
            return content
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def change_address(self, id, street=None, street_no=None, city=None, zip=None):
        """Method changes address

        Args:
           id (int): address id
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code    

        Returns:
           bool: result

        """

        msg = 'id:{0}, street:{1}, street_no:{2}, city:{3}, zip:{4}'.format(
            id, street, street_no, city, zip)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'change_address', msg),
                      self._mh.fromhere())

        url = self._url + 'address'
        address = Address(id, street, street_no, city, zip)
        body = address.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_changed', 'address', id),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def assign_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):
        """Method assigns address role

        Args:
           id (int): address id
           role (str): role title, lov_address_role.title
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id  

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, contact:{2}, customer:{3}, payer:{4}, subscriber:{5}'.format(
            id, role, contact, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'assign_address_role', msg),
                      self._mh.fromhere())

        url = self._url + 'address/role'
        address_role = AddressRole(
            id, role, contact, customer, payer, subscriber)
        body = address_role.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_role_assigned', 'address'),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def revoke_address_role(self, id, role, contact=None, customer=None, payer=None, subscriber=None):
        """Method revokes address role

        Args:
           id (int): address id
           role (str): role title, lov_address_role.title
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id  

        Returns:
           bool: result

        """

        msg = 'id:{0}, role:{1}, contact:{2}, customer:{3}, payer:{4}, subscriber:{5}'.format(
            id, role, contact, customer, payer, subscriber)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'revoke_address_role', msg),
                      self._mh.fromhere())

        url = self._url + 'address/role'
        address_role = AddressRole(
            id, role, contact, customer, payer, subscriber)
        body = address_role.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_role_revoked', 'address'),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def read_services(self, customer=None, payer=None, subscriber=None, service=None):
        """Method read services

        Args:
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           service (int): service id, lov_service.id, default empty, read all services for entity     

        Returns:
           list: list of crm_entities.Service

        """

        msg = 'customer:{0}, payer:{1}, subscriber:{2}, service:{3}'.format(
            customer, payer, subscriber, service)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'read_services', msg),
                      self._mh.fromhere())

        params = {}
        if (customer != None):
            params['customer'] = customer
        if (payer != None):
            params['payer'] = payer
        if (subscriber != None):
            params['subscriber'] = subscriber
        if (service != None):
            params['service'] = service

        url = self._url + 'service'
        status, content = self._client.send_request(
            url, method='GET', params=params)

        if (status == 200):

            doc = loads(content)

            services = []
            for service in doc['services']['service']:

                params = {}
                for param in service['params']['entry']:
                    params[param['key']] = param['value']

                services.append(
                    Service(service['id'], service['name'], service['status'], params))

            for service in services:
                self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_found', 'service', service),
                              self._mh.fromhere())

            return services

        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return None

    def create_service(self, service, customer=None, payer=None, subscriber=None, status='active', params={}):
        """Method creates service

        Args: 
           service (int): service id, lov_service.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): status, lov_status.title, default active
           params (dict): key (int), lov_service_param.id, value (str)  

        Returns:
           bool: result

        """

        msg = 'service:{0}, customer:{1}, payer:{2}, subscriber:{3}, status:{4}, params:{5}'.format(
            service, customer, payer, subscriber, status, params)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'create_service', msg),
                      self._mh.fromhere())

        url = self._url + 'service'
        service_operation = ServiceOperation(
            service, customer, payer, subscriber, status, params)
        body = service_operation.tojson()
        status, content = self._client.send_request(
            url, method='POST', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_created', 'service', service),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False

    def change_service(self, service, customer=None, payer=None, subscriber=None, status=None, params={}):
        """Method changes service

        Args: 
           service (int): service id, lov_service.id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): status, lov_status.title, default active
           params (dict): key (int), lov_service_param.id, value (str)  

        Returns:
           bool: result

        """

        msg = 'service:{0}, customer:{1}, payer:{2}, subscriber:{3}, status:{4}, params:{5}'.format(
            service, customer, payer, subscriber, status, params)
        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_func', 'change_service', msg),
                      self._mh.fromhere())

        url = self._url + 'service'
        service_operation = ServiceOperation(
            service, customer, payer, subscriber, status, params)
        body = service_operation.tojson()
        status, content = self._client.send_request(
            url, method='PUT', body=body, content_type='json')

        if (status == 200):
            self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_rest_entity_changed', 'service', service),
                          self._mh.fromhere())
            return True
        else:
            self._mh.dmsg('htk_on_extension_error', 'status:{0}, content:{1}'.format(
                status, content), self._mh.fromhere())
            return False



# -*- coding: utf-8 -*-
"""Web server

.. module:: testenv.web_server
   :platform: Unix
   :synopsis: Web server
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core.masterhead import MasterHead
from hydratk.extensions.testenv.rest_handler import RestHandler
from hydratk.extensions.testenv.soap_handler import SoapHandler
import hydratk.lib.system.config as syscfg
from web import application, httpserver, input, data, NotFound, header, ctx
from os import path

urls = (
    '/', 'Index',
    '/rs/customer', 'Customer',
    '/rs/payer', 'Payer',
    '/rs/subscriber', 'Subscriber',
    '/rs/contact', 'Contact',
    '/rs/contact/role', 'ContactRole',
    '/rs/address', 'Address',
    '/rs/address/role', 'AddressRole',
    '/rs/service', 'Service',
    '/ws/crm', 'SoapService'
)

mh = None


class Server(object):
    """Class Server
    """

    _server = None

    def __init__(self):
        """Class constructor

        Called when the object is initialized    

        Args:
           none

        """

        global mh
        mh = MasterHead.get_head()

    def _start(self):
        """Method starts web server   

        Args:
           none

        Returns:
           void       

        """

        ip = mh.cfg['Extensions']['TestEnv']['server_ip']
        port = mh.cfg['Extensions']['TestEnv']['server_port']
        self._server = application(urls, globals())
        httpserver.runsimple(self._server.wsgifunc(), (str(ip), port))


class Index(object):

    def GET(self):

        return 'Hello, World'


class Customer(object):
    """Handles requests on /rs/customer                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def GET(self):

        return self._rest.read_customer(input())

    def POST(self):

        return self._rest.create_customer(data())

    def PUT(self):

        return self._rest.change_customer(data())


class Payer(object):
    """Handles requests on /rs/payer                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def GET(self):

        return self._rest.read_payer(input())

    def POST(self):

        return self._rest.create_payer(data())

    def PUT(self):

        return self._rest.change_payer(data())


class Subscriber(object):
    """Handles requests on /rs/subscriber                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def GET(self):

        return self._rest.read_subscriber(input())

    def POST(self):

        return self._rest.create_subscriber(data())

    def PUT(self):

        return self._rest.change_subscriber(data())


class Contact(object):
    """Handles requests on /rs/contact                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def GET(self):

        return self._rest.read_contact(input())

    def POST(self):

        return self._rest.create_contact(data())

    def PUT(self):

        return self._rest.change_contact(data())


class ContactRole(object):
    """Handles requests on /rs/contact/role                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def POST(self):

        return self._rest.assign_contact_role(data())

    def PUT(self):

        return self._rest.revoke_contact_role(data())


class Address(object):
    """Handles requests on /rs/address                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def GET(self):

        return self._rest.read_address(input())

    def POST(self):

        return self._rest.create_address(data())

    def PUT(self):

        return self._rest.change_address(data())


class AddressRole(object):
    """Handles requests on /rs/address/role                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def POST(self):

        return self._rest.assign_address_role(data())

    def PUT(self):

        return self._rest.revoke_address_role(data())


class Service(object):
    """Handles requests on /rs/service                           
    """

    _rest = None

    def __init__(self):

        self._rest = RestHandler()

    def GET(self):

        return self._rest.read_services(input())

    def POST(self):

        return self._rest.create_service(data())

    def PUT(self):

        return self._rest.change_service(data())


class SoapService(object):
    """Handles requests on /ws/crm                           
    """

    _soap = None

    def __init__(self):

        self._soap = SoapHandler()

    def GET(self):

        key = list(input().keys())[0]
        ext_dir = mh.cfg['Extensions']['TestEnv']['ext_dir'].format(var_dir=syscfg.HTK_VAR_DIR)

        if (key == 'wsdl'):
            soap_file = path.join(ext_dir, 'crm.wsdl')
        elif (key == 'xsd'):
            soap_file = path.join(ext_dir, 'crm.xsd')
        else:
            return NotFound()

        if (path.exists(soap_file)):
            with open(soap_file, 'r') as file:
                header('Content-Type', 'text/xml')
                return file.read()
        else:
            mh.dmsg('htk_on_extension_error', self._mh._trn.msg('te_web_unknown_file', soap_file),
                    self._mh.fromhere())
            return NotFound()

    def POST(self):

        return self._soap.dispatcher(ctx.env, data())



# -*- coding: utf-8 -*-
"""CRM entity classes

.. module:: testenv.entities.crm_entities
   :platform: Unix
   :synopsis: CRM entity classes
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from lxml.etree import Element, SubElement
from simplejson import dumps


class Customer(object):
    """Class Customer
    """

    def __init__(self, id, name, status, segment, birth_no=None, reg_no=None, tax_no=None):
        """Class constructor

        Customer entity   

        Args:
           id (int): customer id
           name (str): name
           status (str): status, active|deactive|suspend
           segment (int): segment id, 2|3|4|5 RES|VSE|SME|LE
           birth_no (str): birth number
           reg_no (str): registration number
           tax_no (str): tax identification number

        """

        self.id = id
        self.name = name
        self.status = status
        self.segment = segment
        self.birth_no = birth_no
        self.reg_no = reg_no
        self.tax_no = tax_no

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|name:{1}|status:{2}|segment:{3}'.format(self.id, self.name, self.status, self.segment) + \
            u'|birth_no:{0}|reg_no:{1}|tax_no:{2}'.format(
                self.birth_no, self.reg_no, self.tax_no)
        return s

    def toxml(self):
        """Method serializes customer to XML           
        """

        root = Element('customer')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'name').text = self.name
        SubElement(root, 'status').text = self.status
        SubElement(root, 'segment').text = str(self.segment)
        if (self.birth_no != None):
            SubElement(root, 'birth_no').text = self.birth_no
        if (self.reg_no != None):
            SubElement(root, 'reg_no').text = self.reg_no
        if (self.tax_no != None):
            SubElement(root, 'tax_no').text = self.tax_no

        return root

    def tojson(self):
        """Method serializes customer to JSON           
        """

        root = {}

        root['id'] = self.id
        root['name'] = self.name
        root['status'] = self.status
        root['segment'] = self.segment
        if (self.birth_no != None):
            root['birth_no'] = self.birth_no
        if (self.reg_no != None):
            root['reg_no'] = self.reg_no
        if (self.tax_no != None):
            root['tax_no'] = self.tax_no

        return dumps(root)


class Payer(object):
    """Class Payer
    """

    def __init__(self, id, name, status, billcycle, customer, bank_account=None):
        """Class constructor

        Payer entity   

        Args:
           id (int): payer id
           name (str): name
           status (str): status, active|deactive|suspend
           billcycle (int): billcycle id, 1|2|3|4 51|52|53|54        
           bank_account (str): banking account
           customer (id): assigned customer id           

        """

        self.id = id
        self.name = name
        self.status = status
        self.billcycle = billcycle
        self.bank_account = bank_account
        self.customer = customer

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|name:{1}|status:{2}|billcycle:{3}'.format(self.id, self.name, self.status, self.billcycle) + \
            u'|bank_account:{0}|customer:{1}'.format(
                self.bank_account, self.customer)
        return s

    def toxml(self):
        """Method serializes payer to XML           
        """

        root = Element('payer')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'name').text = self.name
        SubElement(root, 'status').text = self.status
        SubElement(root, 'billcycle').text = str(self.billcycle)
        if (self.bank_account != None):
            SubElement(root, 'bank_account').text = self.bank_account
        SubElement(root, 'customer').text = str(self.customer)

        return root

    def tojson(self):
        """Method serializes payer to JSON           
        """

        root = {}

        root['id'] = self.id
        root['name'] = self.name
        root['status'] = self.status
        root['billcycle'] = self.billcycle
        if (self.bank_account != None):
            root['bank_account'] = self.bank_account
        root['customer'] = self.customer

        return dumps(root)


class Subscriber(object):
    """Class Subscriber
    """

    def __init__(self, id, name, msisdn, status, market, tariff, customer, payer):
        """Class constructor

        Subscriber entity   

        Args:
           id (int): subscriber id           
           name (str): name
           msisdn (str): MSISDN
           status (str): status, active|deactive|suspend
           market (int): market id, 1|2|3 GSM|DSL|FIX
           tariff (int): tariff id, 433|459|434|460
           customer (int): assigned customer id    
           payer (int): assigned payer id       

        """

        self.id = id
        self.name = name
        self.msisdn = msisdn
        self.status = status
        self.market = market
        self.tariff = tariff
        self.customer = customer
        self.payer = payer

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|name:{1}|msisdn:{2}|status:{3}'.format(self.id, self.name, self.msisdn, self.status) + \
            u'|market:{0}|tariff:{1}|customer:{2}|payer:{3}'.format(
                self.market, self.tariff, self.customer, self.payer)
        return s

    def toxml(self):
        """Method serializes subscriber to XML           
        """

        root = Element('subscriber')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'name').text = self.name
        SubElement(root, 'msisdn').text = self.msisdn
        SubElement(root, 'status').text = self.status
        SubElement(root, 'market').text = str(self.market)
        SubElement(root, 'tariff').text = str(self.tariff)
        SubElement(root, 'customer').text = str(self.customer)
        SubElement(root, 'payer').text = str(self.payer)

        return root

    def tojson(self):
        """Method serializes subscriber to JSON           
        """

        root = {}

        root['id'] = self.id
        root['name'] = self.name
        root['msisdn'] = self.msisdn
        root['status'] = self.status
        root['market'] = self.market
        root['tariff'] = self.tariff
        root['customer'] = self.customer
        root['payer'] = self.payer

        return dumps(root)


class Contact(object):
    """Class Contact
    """

    def __init__(self, id, name, phone=None, email=None, roles=[]):
        """Class constructor

        Contact entity   

        Args:
           id (int): contact id           
           name (str): name 
           phone (str): phone number
           email (str): email
           roles (list): contact roles   

        """

        self.id = id
        self.name = name
        self.phone = phone
        self.email = email
        self.roles = roles

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|name:{1}|phone:{2}|email:{3}|roles#'.format(
            self.id, self.name, self.phone, self.email)

        if (len(self.roles) > 0):

            for role in self.roles:
                s += '{0}#'.format(role)

        return s

    def toxml(self):
        """Method serializes contact to XML           
        """

        root = Element('contact')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'name').text = self.name
        if (self.phone != None):
            SubElement(root, 'phone').text = self.phone
        if (self.email != None):
            SubElement(root, 'email').text = self.email

        if (len(self.roles) > 0):
            elem = SubElement(root, 'roles')

            for role in self.roles:
                elem.append(role.toxml())

        return root

    def tojson(self):
        """Method serializes contact to JSON           
        """

        root = {}

        root['id'] = self.id
        root['name'] = self.name
        if (self.phone != None):
            root['phone'] = self.phone
        if (self.email != None):
            root['email'] = self.email

        if (len(self.roles) > 0):
            el_roles = []

            for role in self.roles:
                el_role = {'id': role.id, 'title': role.title,
                           'customer': role.customer, 'payer': role.payer,
                           'subscriber': role.subscriber}
                el_roles.append(el_role)

            root['roles'] = {'role': el_roles}

        return dumps(root)


class ContactRole(object):
    """Class ContactRole
    """

    def __init__(self, id, title, customer=None, payer=None, subscriber=None):
        """Class constructor

        Contact role entity   

        Args:
           id (int): contact id           
           title (str): role title, contract|contact|invoicing
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id 

        """

        self.id = id
        self.title = title
        self.customer = customer
        self.payer = payer
        self.subscriber = subscriber

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|title:{1}|customer:{2}|payer:{3}'.format(self.id, self.title, self.customer, self.payer) + \
            u'|subscriber:{0}'.format(self.subscriber)
        return s

    def toxml(self):
        """Method serializes contact role to XML           
        """

        root = Element('role')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'title').text = self.title
        if (self.customer != None):
            SubElement(root, 'customer').text = str(self.customer)
        if (self.payer != None):
            SubElement(root, 'payer').text = str(self.payer)
        if (self.subscriber != None):
            SubElement(root, 'subscriber').text = str(self.subscriber)

        return root

    def tojson(self):
        """Method serializes contact role to JSON           
        """

        root = {}

        root['id'] = self.id
        root['title'] = self.title
        if (self.customer != None):
            root['customer'] = self.customer
        if (self.payer != None):
            root['payer'] = self.payer
        if (self.subscriber != None):
            root['subscriber'] = self.subscriber

        return dumps(root)


class Address(object):
    """Class Address
    """

    def __init__(self, id, street, street_no, city, zip, roles={}):
        """Class constructor

        Address role entity   

        Args:
           id (int): address id           
           street (str): street
           street_no (str): street number
           city (str): city
           zip (int): zip code
           roles (list): address roles

        """

        self.id = id
        self.street = street
        self.street_no = street_no
        self.city = city
        self.zip = zip
        self.roles = roles

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|street:{1}|street_no:{2}|city:{3}'.format(self.id, self.street, self.street_no, self.city) + \
            u'|zip:{0}|roles#'.format(self.zip)

        if (len(self.roles) > 0):

            for role in self.roles:
                s += '{0}#'.format(role)

        return s

    def toxml(self):
        """Method serializes address to XML           
        """

        root = Element('address')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'street').text = self.street
        SubElement(root, 'street_no').text = self.street_no
        SubElement(root, 'city').text = self.city
        SubElement(root, 'zip').text = str(self.zip)

        if (len(self.roles) > 0):
            elem = SubElement(root, 'roles')

            for role in self.roles:
                elem.append(role.toxml())

        return root

    def tojson(self):
        """Method serializes address to JSON           
        """

        root = {}

        root['id'] = self.id
        root['street'] = self.street
        root['street_no'] = self.street_no
        root['city'] = self.city
        root['zip'] = self.zip

        if (len(self.roles) > 0):
            el_roles = []

            for role in self.roles:
                el_role = {'id': role.id, 'title': role.title,
                           'contact': role.contact, 'customer': role.customer,
                           'payer': role.payer, 'subscriber': role.subscriber}
                el_roles.append(el_role)

            root['roles'] = {'role': el_roles}

        return dumps(root)


class AddressRole(object):
    """Class AddressRole
    """

    def __init__(self, id, title, contact=None, customer=None, payer=None, subscriber=None):
        """Class constructor

        Address role entity   

        Args:
           id (int): address id           
           title (str): role title, contract|contact|invoicing|delivery
           contact (int): assigned contact id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id 

        """

        self.id = id
        self.title = title
        self.contact = contact
        self.customer = customer
        self.payer = payer
        self.subscriber = subscriber

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|title:{1}|contact:{2}|customer:{3}'.format(self.id, self.title, self.contact, self.customer) + \
            u'|payer:{0}|subscriber:{1}'.format(self.payer, self.subscriber)
        return s

    def toxml(self):
        """Method serializes address role to XML           
        """

        root = Element('role')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'title').text = self.title
        if (self.contact != None):
            SubElement(root, 'contact').text = str(self.contact)
        if (self.customer != None):
            SubElement(root, 'customer').text = str(self.customer)
        if (self.payer != None):
            SubElement(root, 'payer').text = str(self.payer)
        if (self.subscriber != None):
            SubElement(root, 'subscriber').text = str(self.subscriber)

        return root

    def tojson(self):
        """Method serializes address role to JSON           
        """

        root = {}

        root['id'] = self.id
        root['title'] = self.title
        if (self.contact != None):
            root['contact'] = self.contact
        if (self.customer != None):
            root['customer'] = self.customer
        if (self.payer != None):
            root['payer'] = self.payer
        if (self.subscriber != None):
            root['subscriber'] = self.subscriber

        return dumps(root)


class Service(object):
    """Class Service
    """

    def __init__(self, id, name, status, params={}):
        """Class constructor

        Service entity   

        Args:
           id (int): service id           
           name (str): name
           status (str): status, active|deactive|suspend
           params (dict): parameters

        """

        self.id = id
        self.name = name
        self.status = status
        self.params = params

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'id:{0}|name:{1}|status:{2}|params#'.format(
            self.id, self.name, self.status)

        if (len(self.params) > 0):

            for key, value in self.params.items():
                s += '{0}:{1}#'.format(key, value)

        return s

    def toxml(self):
        """Method serializes service to XML           
        """

        root = Element('service')

        SubElement(root, 'id').text = str(self.id)
        SubElement(root, 'name').text = self.name
        SubElement(root, 'status').text = self.status

        elem = SubElement(root, 'params')
        for key, value in self.params.items():
            el_param = Element('entry')
            SubElement(el_param, 'key').text = str(key)
            SubElement(el_param, 'value').text = value
            elem.append(el_param)

        return root

    def tojson(self):
        """Method serializes service to JSON           
        """

        root = {}

        root['id'] = self.id
        root['name'] = self.name
        root['status'] = self.status

        el_params = []
        for key, value in self.params.items():
            param = {}
            param['key'] = key
            param['value'] = value
            el_params.append(param)

        root['params'] = {'entry': el_params}

        return dumps(root)


class ServiceOperation(object):
    """Class ServiceOperation
    """

    def __init__(self, service, customer=None, payer=None, subscriber=None, status=None, params={}):
        """Class constructor

        Service operation entity   

        Args:
           service (int): service id
           customer (int): assigned customer id
           payer (int): assigned payer id
           subscriber (int): assigned subscriber id
           status (str): service status, active|deactive|suspend
           params (dict): service parameters

        """

        self.service = service
        self.customer = customer
        self.payer = payer
        self.subscriber = subscriber
        self.status = status
        self.params = params

    def __str__(self):
        """Method overrides __str__         
        """

        s = u'service:{0}|customer:{1}|payer:{2}'.format(self.service, self.customer, self.payer) + \
            u'|subscriber:{0}|status:{1}|params#'.format(
                self.subscriber, self.status)

        if (len(self.params) > 0):

            for key, value in self.params.items():
                s += '{0}:{1}#'.format(key, value)

        return s

    def toxml(self):
        """Method serializes service operation to XML           
        """

        root = Element('operation')

        SubElement(root, 'service').text = str(self.service)
        if (self.customer != None):
            SubElement(root, 'customer').text = str(self.customer)
        if (self.payer != None):
            SubElement(root, 'payer').text = str(self.payer)
        if (self.subscriber != None):
            SubElement(root, 'subscriber').text = str(self.subscriber)
        if (self.status != None):
            SubElement(root, 'status').text = self.status

        elParams = SubElement(root, 'params')

        for key, value in self.params.items():
            elParam = SubElement(elParams, 'entry')
            elem = SubElement(elParam, 'key')
            elem.text = str(key)
            elem = SubElement(elParam, 'value')
            elem.text = str(value)

        return root

    def tojson(self):
        """Method serializes service operation to JSON           
        """

        root = {}

        root['service'] = self.service
        if (self.customer != None):
            root['customer'] = self.customer
        if (self.payer != None):
            root['payer'] = self.payer
        if (self.subscriber != None):
            root['subscriber'] = self.subscriber
        if (self.status != None):
            root['status'] = self.status

        el_params = []
        for key, value in self.params.items():
            param = {}
            param['key'] = key
            param['value'] = value
            el_params.append(param)

        root['params'] = {'entry': el_params}

        return dumps(root)



# -*- coding: utf-8 -*-
from setuptools import setup as st_setup
from setuptools import find_packages as st_find_packages
from sys import argv, version_info
from pkgutil import find_loader
import hydratk.lib.install.task as task
import hydratk.lib.system.config as syscfg

try:
    os_info = syscfg.get_supported_os()
except Exception as exc:
    print(str(exc))
    exit(1)

with open("README.rst", "r") as f:
    readme = f.read()

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Console",
    "Environment :: Other Environment",
    "Intended Audience :: Developers",
    "License :: Freely Distributable",
    "Operating System :: OS Independent",
    "License :: OSI Approved :: BSD License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: Implementation",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Software Development :: Libraries :: Application Frameworks",
    "Topic :: Utilities"
]


def version_update(cfg, *args):

    if (version_info[0] == 3):
        if (find_loader('web') == None):
            cfg['modules'][-1] = {'module': 'git+https://github.com/webpy/webpy.git@py3#egg=webpy'}
        else:
            del cfg['modules'][-1]

config = {
    'pre_tasks': [
        version_update,
        task.install_modules
    ],

    'post_tasks': [
        task.set_config,
        task.copy_files,
        task.set_access_rights,
        task.set_manpage
    ],

    'modules': [
        {'module': 'hydratk', 'version': '>=0.5.0'},
        {'module': 'hydratk-ext-yoda',    'version': '>=0.2.2'},
        {'module': 'hydratk-lib-network', 'version': '>=0.2.0'},
        {'module': 'web.py',              'version': '>=0.37'}
    ],

    'files': {
        'config': {
            'etc/hydratk/conf.d/hydratk-ext-testenv.conf': '{0}/hydratk/conf.d'.format(syscfg.HTK_ETC_DIR)
        },
        'data': {
            'var/local/hydratk/testenv/install_db.sql': '{0}/hydratk/testenv'.format(syscfg.HTK_VAR_DIR),
            'var/local/hydratk/testenv/crm.wsdl': '{0}/hydratk/testenv'.format(syscfg.HTK_VAR_DIR),
            'var/local/hydratk/testenv/crm.xsd': '{0}/hydratk/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yodalib/hydratk/__init__.py': '{0}/hydratk/yoda/lib/yodalib/hydratk'.format(syscfg.HTK_VAR_DIR),
            'tests/yodalib/hydratk/extensions/__init__.py': '{0}/hydratk/yoda/lib/yodalib/hydratk/extensions'.format(syscfg.HTK_VAR_DIR),
            'tests/yodalib/hydratk/extensions/testenv/__init__.py': '{0}/hydratk/yoda/lib/yodalib/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yodalib/hydratk/extensions/testenv/db_int.py': '{0}/hydratk/yoda/lib/yodalib/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yodalib/hydratk/extensions/testenv/rest_int.py': '{0}/hydratk/yoda/lib/yodalib/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yodalib/hydratk/extensions/testenv/soap_int.py': '{0}/hydratk/yoda/lib/yodalib/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yodahelpers/hydratk/__init__.py': '{0}/hydratk/yoda/helpers/yodahelpers/hydratk'.format(syscfg.HTK_VAR_DIR),
            'tests/yodahelpers/hydratk/extensions/__init__.py': '{0}/hydratk/yoda/helpers/yodahelpers/hydratk/extensions'.format(syscfg.HTK_VAR_DIR),
            'tests/yodahelpers/hydratk/extensions/testenv/__init__.py': '{0}/hydratk/yoda/helpers/yodahelpers/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yodahelpers/hydratk/extensions/testenv/helpers.py': '{0}/hydratk/yoda/helpers/yodahelpers/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yoda-tests/hydratk/extensions/testenv/db.jedi': '{0}/hydratk/yoda/yoda-tests/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yoda-tests/hydratk/extensions/testenv/rest.jedi': '{0}/hydratk/yoda/yoda-tests/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR),
            'tests/yoda-tests/hydratk/extensions/testenv/soap.jedi': '{0}/hydratk/yoda/yoda-tests/hydratk/extensions/testenv'.format(syscfg.HTK_VAR_DIR)
        },
        'manpage': 'doc/testenv.1'
    },

    'rights': {
        '{0}/hydratk'.format(syscfg.HTK_VAR_DIR): 'a+rwx'
    }
}

task.run_pre_install(argv, config)

entry_points = {
    'console_scripts': [
        'testenv = hydratk.extensions.testenv.bootstrapper:run_app'
    ]
}

st_setup(
    name='hydratk-ext-testenv',
    version='0.2.2',
    description='Test environment for test automation exercises',
    long_description=readme,
    author='Petr Rašek, HydraTK team',
    author_email='bowman@hydratk.org, team@hydratk.org',
    url='http://extensions.hydratk.org/testenv',
    license='BSD',
    packages=st_find_packages('src'),
    package_dir={'': 'src'},
    classifiers=classifiers,
    zip_safe=False,
    entry_points=entry_points,
    keywords='hydratk,testing,test environment,web server,database',
    requires_python='>=2.6,!=3.0.*,!=3.1.*,!=3.2.*',
    platforms='Linux'
)

task.run_post_install(argv, config)



# -*- coding: utf-8 -*-
"""Providing custom bootstrapper for testenv standalone app

.. module:: testenv.bootstrapper
   :platform: Unix
   :synopsis: Providing custom bootstrapper for testenv standalone app
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

import sys

PYTHON_MAJOR_VERSION = sys.version_info[0]
if PYTHON_MAJOR_VERSION == 2:
    reload(sys)
    sys.setdefaultencoding('UTF8')


def run_app():
    """Methods runs testenv standalone application

    Args:
       none

    Returns:
       void

    """

    from hydratk.core.masterhead import MasterHead
    mh = MasterHead.get_head()
    mh.set_cli_cmdopt_profile('testenv')
    mh.run_fn_hook('h_bootstrap')
    trn = mh.get_translator()
    mh.dmsg('htk_on_debug_info', trn.msg('htk_app_exit'), mh.fromhere())
    sys.exit(0)



# -*- coding: utf-8 -*-
"""Handles SOAP operations

.. module:: testenv.soap_handler
   :platform: Unix
   :synopsis: Handles SOAP operations
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""

from hydratk.core.masterhead import MasterHead
from hydratk.extensions.testenv.db_handler import DbHandler
from lxml.etree import Element, SubElement, tostring, XMLSyntaxError
from lxml import objectify


class SoapHandler(object):
    """Class SoapHandler
    """

    _mh = None
    _nsmap = None
    _ns0 = None
    _ns1 = None

    def __init__(self):
        """Class constructor

        Called when the object is initialized   

        Args: 
           none

        """

        self._mh = MasterHead.get_head()
        self._nsmap = {'soapenv': 'http://www.w3.org/2003/05/soap-envelope/',
                       'ns0': 'http://hydratk.org/'}
        self._ns0 = '{%s}' % self._nsmap['soapenv']
        self._ns1 = '{%s}' % self._nsmap['ns0']

    def _get_db(self):
        """Method connect to database     

        Args:
           none

        Returns:
           DB_INT: DB client                 

        """

        db = DbHandler()
        db.connect()
        return db

    def _fault(self, message):
        """Method creates SOAP fault     

        Args:
           message (str): error message 

        Returns:
           str: SOAP fault               

        """

        root = Element(self._ns0 + 'Envelope', nsmap=self._nsmap)

        SubElement(root, self._ns0 + 'Header')
        body = SubElement(root, self._ns0 + 'Body')
        fault = SubElement(body, self._ns0 + 'Fault')
        SubElement(fault, self._ns1 + 'message').text = str(message)

        return tostring(root, pretty_print=True)

    def _response(self, method, content):
        """Method creates SOAP response     

        Args:
           method (str): method name
           content (xml): response body

        Returns:
           str: SOAP response                  

        """

        root = Element(self._ns0 + 'Envelope', nsmap=self._nsmap)

        SubElement(root, self._ns0 + 'Header')
        body = SubElement(root, self._ns0 + 'Body')
        response = SubElement(body, self._ns1 + method + '_response')
        response.append(content)

        return tostring(root, pretty_print=True)

    def dispatcher(self, headers, data):
        """Method dispatches request according to SOAPAction        

        Args:
           headers (dict): HTTP headers
           data (str): SOAP request   

        Returns:
           str: SOAP response or fault               

        """

        try:

            action = headers[
                'HTTP_SOAPACTION'] if 'HTTP_SOAPACTION' in headers else None
            doc = objectify.fromstring(data)
            el_action = self._ns1 + action

            if (action == None):
                return self._fault('Missing SOAPAction')
            elif (action == 'read_customer'):
                return self.read_customer(doc.Body[el_action])
            elif (action == 'create_customer'):
                return self.create_customer(doc.Body[el_action])
            elif (action == 'change_customer'):
                return self.change_customer(doc.Body[el_action])
            elif (action == 'read_payer'):
                return self.read_payer(doc.Body[el_action])
            elif (action == 'create_payer'):
                return self.create_payer(doc.Body[el_action])
            elif (action == 'change_payer'):
                return self.change_payer(doc.Body[el_action])
            elif (action == 'read_subscriber'):
                return self.read_subscriber(doc.Body[el_action])
            elif (action == 'create_subscriber'):
                return self.create_subscriber(doc.Body[el_action])
            elif (action == 'change_subscriber'):
                return self.change_subscriber(doc.Body[el_action])
            elif (action == 'read_contact'):
                return self.read_contact(doc.Body[el_action])
            elif (action == 'create_contact'):
                return self.create_contact(doc.Body[el_action])
            elif (action == 'change_contact'):
                return self.change_contact(doc.Body[el_action])
            elif (action == 'assign_contact_role'):
                return self.assign_contact_role(doc.Body[el_action])
            elif (action == 'revoke_contact_role'):
                return self.revoke_contact_role(doc.Body[el_action])
            elif (action == 'read_address'):
                return self.read_address(doc.Body[el_action])
            elif (action == 'create_address'):
                return self.create_address(doc.Body[el_action])
            elif (action == 'change_address'):
                return self.change_address(doc.Body[el_action])
            elif (action == 'assign_address_role'):
                return self.assign_address_role(doc.Body[el_action])
            elif (action == 'revoke_address_role'):
                return self.revoke_address_role(doc.Body[el_action])
            elif (action == 'read_services'):
                return self.read_services(doc.Body[el_action])
            elif (action == 'create_service'):
                return self.create_service(doc.Body[el_action])
            elif (action == 'change_service'):
                return self.change_service(doc.Body[el_action])
            else:
                return self._fault('Invalid SOAPAction {0}'.format(action))

        except XMLSyntaxError as ex:
            self._mh.dmsg(
                'htk_on_extension_error', 'XML error: {0}'.format(ex), self._mh.fromhere())
            return self._fault('Invalid XML - ' + ex)

    def read_customer(self, doc):
        """Method handles read_customer request         

        Args:
           doc (xml): read_customer request with customer id 

        Returns:
           xml: read_customer_response with customer detail,
                SOAP fault when customer not found   

        Example:

        .. code-block:: xml

           <read_customer>
             <id>1</id>
           </read_customer>         

           <read_customer_response>
             <customer>
               <id>1</id>
               <name>Charlie Bowman</name>
               <status>active</status>
               <segment>2</segment>
               <birth_no>700101/0001</birth_no>
               <reg_no>123456</reg_no>
               <tax_no>CZ123456</tax_no>
             </customer>
           </read_customer_response>                       

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'read_customer', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None

        db = self._get_db()
        customer = db.read_customer(id)
        db.disconnect()

        if (customer != None):
            return self._response('read_customer', customer.toxml())
        else:
            return self._fault('Customer {0} not found'.format(id))

    def create_customer(self, doc):
        """Method handles create_customer request         

        Args:
           doc (xml): create_customer request

        Returns:
           xml: create_customer_response with id of created customer,
                SOAP fault when customer not created 

        Example:

        .. code-block:: xml

           <create_customer>
             <name>Charlie Bowman</name>
             <status>active</status>
             <segment>2</segment>
             <birth_no>700101/0001</birth_no>
             <reg_no>123456</reg_no>
             <tax_no>CZ123456</tax_no>
           </create_customer>  

           <create_customer_response>
             <id>1</id>
           <create_customer_response>                  

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'create_customer', tostring(doc)),
                      self._mh.fromhere())

        name = doc.find('name').text if doc.find('name') else None
        status = doc.find('status').text if doc.find('status') else None
        segment = doc.find('segment').text if doc.find('segment') else None
        birth_no = doc.find('birth_no').text if doc.find('birth_no') else None
        reg_no = doc.find('reg_no').text if doc.find('reg_no') else None
        tax_no = doc.find('tax_no').text if doc.find('tax_no') else None

        db = self._get_db()
        id = db.create_customer(
            name, segment, status, birth_no, reg_no, tax_no)
        db.disconnect()

        if (id != None):
            elem = Element('id')
            elem.text = str(id)
            return self._response('create_customer', elem)
        else:
            return self._fault('Customer not created')

    def change_customer(self, doc):
        """Method handles change_customer request         

        Args:
           doc (xml): change_customer request

        Returns:
           xml: change_customer_response with result true when customer changed,
                SOAP fault when customer not changed

        Example:

        .. code-block:: xml

           <change_customer>
             <id>1</id>
             <name>Charlie Bowman</name>
             <status>active</status>
             <segment>2</segment>
             <birth_no>700101/0001</birth_no>
             <reg_no>123456</reg_no>
             <tax_no>CZ123456</tax_no>
           </create_customer>  

           <change_customer_response>
             <result>true</true>
           </change_customer_response>                      

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'change_customer', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        name = doc.find('name').text if doc.find('name') else None
        status = doc.find('status').text if doc.find('status') else None
        segment = doc.find('segment').text if doc.find('segment') else None
        birth_no = doc.find('birth_no').text if doc.find('birth_no') else None
        reg_no = doc.find('reg_no').text if doc.find('reg_no') else None
        tax_no = doc.find('tax_no').text if doc.find('tax_no') else None

        db = self._get_db()
        res = db.change_customer(
            id, name, status, segment, birth_no, reg_no, tax_no)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('change_customer', elem)
        else:
            return self._fault('Customer not changed')

    def read_payer(self, doc):
        """Method handles read_payer request         

        Args:
           doc (xml): read_payer request with payer id

        Returns:
           xml: read_payer_response with payer detail,
                SOAP fault when payer not found 

        Example:

        .. code-block:: xml        

           <read_payer>
             <id>1</id>
           </read_payer>

           <read_payer_response>
             <payer>
               <id>1</id>
               <name>Charlie Bowman</name>
               <status>active</status>
               <billcycle>1</billcycle>
               <bank_account>12345/0100</bank_account>
               <customer>1</customer>
             </payer>
           </read_payer_response>                                    

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'read_payer', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None

        db = self._get_db()
        payer = db.read_payer(id)
        db.disconnect()

        if (payer != None):
            return self._response('read_payer', payer.toxml())
        else:
            return self._fault('Payer {0} not found'.format(id))

    def create_payer(self, doc):
        """Method handles create_payer request         

        Args:
           doc (xml): create_payer request 

        Returns:
           xml: create_payer_response with id of created payer,
                SOAP fault when payer not created

        Example:

        .. code-block:: xml

           <create_payer>
             <name>Charlie Bowman</name>
             <status>active</status>
             <billcycle>1</billcycle>
             <bank_account>12345/0100</bank_account>
             <customer>1</customer>
           </create_payer>

           <create_payer_response>
             <id>1</id>
           </create_payer_response>                    

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'create_payer', tostring(doc)),
                      self._mh.fromhere())

        name = doc.find('name').text if doc.find('name') else None
        status = doc.find('status').text if doc.find('status') else None
        billcycle = doc.find('billcycle').text if doc.find(
            'billcycle') else None
        bank_account = doc.find('bank_account').text if doc.find(
            'bank_account') else None
        customer = doc.find('customer').text if doc.find('customer') else None

        db = self._get_db()
        id = db.create_payer(name, billcycle, customer, status, bank_account)
        db.disconnect()

        if (id != None):
            elem = Element('id')
            elem.text = str(id)
            return self._response('create_payer', elem)
        else:
            return self._fault('Payer not created')

    def change_payer(self, doc):
        """Method handles change_payer request         

        Args:
           doc (xml): change_payer request

        Returns:
           xml: change_payer_response with result true when payer changed,
                SOAP fault when payer not changed 

        Example:

        .. code-block:: xml

           <change_payer>
             <id>1</id>
             <name>Charlie Bowman</name>
             <status>active</status>
             <billcycle>1</billcycle>
             <bank_account>12345/0100</bank_account>
             <customer>1</customer>
           </change_payer>

           <change_payer_response>
             <result>true</result>
           </change_payer_response>                                  

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'change_payer', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        name = doc.find('name').text if doc.find('name') else None
        status = doc.find('status').text if doc.find('status') else None
        billcycle = doc.find('billcycle').text if doc.find(
            'billcycle') else None
        bank_account = doc.find('bank_account').text if doc.find(
            'bank_account') else None
        customer = doc.find('customer').text if doc.find('customer') else None

        db = self._get_db()
        res = db.change_payer(
            id, name, status, billcycle, bank_account, customer)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('change_payer', elem)
        else:
            return self._fault('Payer not changed')

    def read_subscriber(self, doc):
        """Method handles read_csubscriber request         

        Args:
           doc (xml): read_subscriber request with subscriber id 

        Returns:
           xml: read_subscriber_response with subscriber detail,
                SOAP fault when subscriber not found  

        Example:

        .. code-block:: xml        

          <read_subscriber>
            <id>1</id>
          <read_subscriber> 

          <read_subscriber_response>
            <subscriber>
              <id>1</id>
              <name>Charlie Bowman</name>
              <msisdn>12345</msisdn>
              <status>active</status>
              <market>1</market>
              <tariff>433</tariff>
              <customer>1</customer>
              <payer>1</payer>
            </subscriber>
          </read_subscriber_response>                                

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'read_subscriber', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None

        db = self._get_db()
        subscriber = db.read_subscriber(id)
        db.disconnect()

        if (subscriber != None):
            return self._response('read_subscriber', subscriber.toxml())
        else:
            return self._fault('Subscriber {0} not found'.format(id))

    def create_subscriber(self, doc):
        """Method handles create_subscriber request         

        Args:
           doc(xml): create_subscriber request 

        Returns:
           xml: create_subscriber_response with id of created subscriber,
                SOAP fault when subscriber not created   

        Example:

        .. code-block:: xml

          <create_subscriber>
            <name>Charlie Bowman</name>
            <msisdn>12345</msisdn>
            <status>active</status>
            <market>1</market>
            <tariff>433</tariff>
            <customer>1</customer>
            <payer>1</payer>
          </create_subscriber>

          <create_subscriber_response>
            <id>1</id>
          <create_subscriber_response>                

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'create_subscriber', tostring(doc)),
                      self._mh.fromhere())

        name = doc.find('name').text if doc.find('name') else None
        msisdn = doc.find('msisdn').text if doc.find('msisdn') else None
        status = doc.find('status').text if doc.find('status') else None
        market = doc.find('market').text if doc.find('market') else None
        tariff = doc.find('tariff').text if doc.find('tariff') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None

        db = self._get_db()
        id = db.create_subscriber(
            name, msisdn, market, tariff, customer, payer, status)
        db.disconnect()

        if (id != None):
            elem = Element('id')
            elem.text = str(id)
            return self._response('create_subscriber', elem)
        else:
            return self._fault('Subscriber not created')

    def change_subscriber(self, doc):
        """Method handles change_subscriber request         

        Args:
           doc (xml): change_subscriber request

        Returns:
           xml: change_subscriber_response with result true when subscriber changed,
                SOAP fault when subscriber not changed  

        Example:

        .. code-block:: xml

          <change_subscriber>
            <id>1</id>
            <name>Charlie Bowman</name>
            <msisdn>12345</msisdn>
            <status>active</status>
            <market>1</market>
            <tariff>433</tariff>
            <customer>1</customer>
            <payer>1</payer>
          </change_subscriber>

          <change_subscriber_response>
            <result>true</result>
          <change_subscriber_response>                                

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'change_subscriber', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        name = doc.find('name').text if doc.find('name') else None
        msisdn = doc.find('msisdn').text if doc.find('msisdn') else None
        status = doc.find('status').text if doc.find('status') else None
        market = doc.find('market').text if doc.find('market') else None
        tariff = doc.find('tariff').text if doc.find('tariff') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None

        db = self._get_db()
        res = db.change_subscriber(
            id, name, msisdn, status, market, tariff, customer, payer)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('change_subscriber', elem)
        else:
            return self._fault('Subscriber not changed')

    def read_contact(self, doc):
        """Method handles read_contact request         

        Args:
           doc (int): read_contact request with contact id

        Returns:
           xml: read_contact_response with contact detail, choice customer|payer|subscriber,
                SOAP fault when contact not found  

        Example:

        .. code-block:: xml

           <read_contact>
             <id>1</id>
           </read_contact>

           <read_contact_response>   
             <contact> 
               <id>1</id>
               <name>Charlie Bowman</name>
               <phone>12345</phone>
               <email>xxx@xxx.com</email>
               <roles>
                 <role>
                   <id>1</id>
                   <title>contract</title>
                   <customer>1</customer>
                   <payer>1</payer>
                   <subscribe>1</subscriber>
                 </role>
               </roles>
             </contact>
           </read_contact_response>                                  

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'read_contact', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None

        db = self._get_db()
        contact = db.read_contact(id)
        db.disconnect()

        if (contact != None):
            return self._response('read_contact', contact.toxml())
        else:
            return self._fault('Contact {0} not found'.format(id))

    def create_contact(self, doc):
        """Method handles create_contact request         

        Args:
           doc (xml): create_contact request

        Returns:
           xml: create_contact_response with id of created contact,
                SOAP fault when contact not created

        Example:

        .. code-block:: xml

           <create_contact>    
             <name>Charlie Bowman</name>
             <phone>12345</phone>
             <email>xxx@xxx.com</email>
           </create_contact> 

           <create_contact_response>
             <id>1</id>
           </create_contact_response>                                 

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'create_contact', tostring(doc)),
                      self._mh.fromhere())

        name = doc.find('name').text if doc.find('name') else None
        phone = doc.find('phone').text if doc.find('phone') else None
        email = doc.find('email').text if doc.find('email') else None

        db = self._get_db()
        id = db.create_contact(name, phone, email)
        db.disconnect()

        if (id != None):
            elem = Element('id')
            elem.text = str(id)
            return self._response('create_contact', elem)
        else:
            return self._fault('Contact not created')

    def change_contact(self, doc):
        """Method handles change_contact request         

        Args:
           doc (xml): change_contact request

        Returns:
           xml: change_contact_response with result true when contact changed,
                SOAP fault when contact not changed    

        Example:

        .. code-block:: xml

           <change_contact>    
             <id>1</id>
             <name>Charlie Bowman</name>
             <phone>12345</phone>
             <email>xxx@xxx.com</email>
           </change_contact> 

           <change_contact_response>
             <result>true</true>
           </change_contact_response>                     

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'change_contact', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        name = doc.find('name').text if doc.find('name') else None
        phone = doc.find('phone').text if doc.find('phone') else None
        email = doc.find('email').text if doc.find('email') else None

        db = self._get_db()
        res = db.change_contact(id, name, phone, email)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('change_contact', elem)
        else:
            return self._fault('Contact not changed')

    def assign_contact_role(self, doc):
        """Method handles assign_contact_role request         

        Args:
           doc (xml): assign_contact_role request, choice customer|payer|subscriber

        Returns:
           xml: assign_contact_role_response with result true when contact role assigned,
                SOAP fault when contact role not assigned   

        Example:

        .. code-block:: xml

           <assign_contact_role>    
             <id>1</id>
             <title>contract</title>         
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
           </assign_contact_role> 

           <assign_contact_role_response>
             <result>true</true>
           </assign_contact_role_response>                              

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'assign_contact_role', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        title = doc.find('title').text if doc.find('title') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None

        db = self._get_db()
        res = db.assign_contact_role(id, title, customer, payer, subscriber)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('assign_contact_role', elem)
        else:
            return self._fault('Contact role not assigned')

    def revoke_contact_role(self, doc):
        """Method handles revoke_contact_role request         

        Args:
           doc (xml): revoke_contact_role request, choice customer|payer|subscriber 

        Returns:
           xml: revoke_contact_role_response with result true when contact role revoked,
                SOAP fault when contact role not revoked    

        Example:

        .. code-block:: xml

           <revoke_contact_role>    
             <id>1</id>
             <title>contract</title>         
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
           </revoke_contact_role> 

           <revoke_contact_role_response>
             <result>true</true>
           </revoke_contact_role_response>                  

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'revoke_contact_role', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        title = doc.find('title').text if doc.find('title') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None

        db = self._get_db()
        res = db.revoke_contact_role(id, title, customer, payer, subscriber)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('revoke_contact_role', elem)
        else:
            return self._fault('Contact role not revoked')

    def read_address(self, doc):
        """Method handles read_address request         

        Args:
           doc (xml) - read_address request with address id

        Returns:
           xml: read_address_response with address detail, choice contact|customer|payer|subscriber,
                SOAP fault when address not found  

        Example:

        .. code-block:: xml

           <read_address>
             <id>1</id>
           </read_address>           

           <read_address_response>
             <address>
               <id>1</id>
               <street>Tomickova</street>
               <street_no>2144/1</street_no>
               <city>Praha</city>
               <zip>14800</zip>
               <roles>
                 <role>
                   <id>1</id>
                   <title>contract</title>
                   <contact>1</contact>
                   <customer>1</customer>
                   <payer>1</payer>
                   <subscriber>1</subscriber>
                 </role>
               </roles>
             </address>
           </read_address_response>                                  

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'read_address', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None

        db = self._get_db()
        address = db.read_address(id)
        db.disconnect()

        if (address != None):
            return self._response('read_address', address.toxml())
        else:
            return self._fault('Address {0} not found'.format(id))

    def create_address(self, doc):
        """Method handles create_address request         

        Args:
           doc (xml): create_address request

        Returns:
           xml: create_address_response with id of created address,
                SOAP fault when address not created

        Example:

        .. code-block:: xml

           <create_address>
             <street>Tomickova</street>
             <street_no>2144/1</street_no>
             <city>Praha</city>
             <zip>14800</zip>
           </create_address>  

           <create_address_response>
             <id>1</id>
           </create_address_response>         

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'create_address', tostring(doc)),
                      self._mh.fromhere())

        street = doc.find('street').text if doc.find('street') else None
        street_no = doc.find('street_no').text if doc.find(
            'street_no') else None
        city = doc.find('city').text if doc.find('city') else None
        zip = doc.find('zip').text if doc.find('zip') else None

        db = self._get_db()
        id = db.create_address(street, street_no, city, zip)
        db.disconnect()

        if (id != None):
            elem = Element('id')
            elem.text = str(id)
            return self._response('create_address', elem)
        else:
            return self._fault('Address not created')

    def change_address(self, doc):
        """Method handles change_address request         

        Args:
           doc(xml): change_address request

        Returns:
           xml: change_address_response with result true when address changed,
                SOAP fault when address not changed  

        Example:

        .. code-block:: xml

           <change_address>
             <id>1</id>
             <street>Tomickova</street>
             <street_no>2144/1</street_no>
             <city>Praha</city>
             <zip>14800</zip>
           </change_address> 

           <change_address_response>
             <result>true</true>
           </change_address_response>                          

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'change_address', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        street = doc.find('street').text if doc.find('street') else None
        street_no = doc.find('street_no').text if doc.find(
            'street_no') else None
        city = doc.find('city').text if doc.find('city') else None
        zip = doc.find('zip').text if doc.find('zip') else None

        db = self._get_db()
        res = db.change_address(id, street, street_no, city, zip)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('change_address', elem)
        else:
            return self._fault('Address not changed')

    def assign_address_role(self, doc):
        """Method handles assign_address_role request         

        Args:
           doc (xml): assign_address_role request, choice contact|customer|payer|subscriber

        Returns:
           xml: assign_address_role_response with result true when address role assigned,
                SOAP fault when address role not assigned

        Example:

        .. code-block:: xml

           <assign_address_role>    
             <id>1</id>
             <title>contract</title> 
             <contact>1</contact>        
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
           </revoke_address_role> 

           <assign_address_role_response>
             <result>true</true>
           </assign_address_role_response>                

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'assign_address_role', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        title = doc.find('title').text if doc.find('title') else None
        contact = doc.find('contact').text if doc.find('contact') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None

        db = self._get_db()
        res = db.assign_address_role(
            id, title, contact, customer, payer, subscriber)
        db.disconnect()

        if (res):
            elem = Element('result')
            elem.text = 'true'
            return self._response('assign_address_role', elem)
        else:
            return self._fault('Address role not assigned')

    def revoke_address_role(self, doc):
        """Method handles revoke_address_role request         

        Args:
           doc (xml): revoke_address_role request, choice contact|customer|payer|subscriber

        Returns:
           xml: revoke_address_role_response with result true when address role revoked,
                SOAP fault when address role not revoked   

        Example:

        .. code-block:: xml

           <revoke_address_role>    
             <id>1</id>
             <title>contract</title>
             <contact>1</contact>         
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
           </revoke_address_role> 

           <revoke_address_role_response>
             <result>true</true>
           </revoke_address_role_response>                      

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'revoke_address_role', tostring(doc)),
                      self._mh.fromhere())

        id = doc.find('id').text if doc.find('id') else None
        title = doc.find('title').text if doc.find('title') else None
        contact = doc.find('contact').text if doc.find('contact') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None

        db = self._get_db()
        res = db.revoke_address_role(
            id, title, contact, customer, payer, subscriber)
        db.disconnect()

        if (res):
            elem = Element('id')
            elem.text = 'true'
            return self._response('revoke_address_role', elem)
        else:
            return self._fault('Address role not revoked')

    def read_services(self, doc):
        """Method handles read_services request         

        Args:
           doc (xml): read_services request, choice customer|payer|subscriber, all services are read if service empty

        Returns:
           xml: read_services_response with list of services,
                SOAP fault when service not found            

        Example:

        .. code-block:: xml

           <read_services>             
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
             <service>615</service>
           </read_services> 

           <read_services_response>
             <services>
               <service>
                 <id>id</id>
                 <name>Telefonni cislo</name>
                 <status>active</active>
                 <params>
                   <entry>
                     <key>121</key>
                     <value>12345</value>
                   </entry>
                  </params>                 
               </service>
             </services>
           </read_services_response>             

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'read_services', tostring(doc)),
                      self._mh.fromhere())

        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None
        service = doc.find('service').text if doc.find('service') else None

        db = self._get_db()
        services = db.read_services(customer, payer, subscriber, service)
        db.disconnect()

        if (len(services) > 0):

            elem = Element('services')
            for service in services:
                elem.append(service.toxml())

            return self._response('read_services', elem)
        else:
            return self._fault('Service not found'.format(id))

    def create_service(self, doc):
        """Method handles create_service request         

        Args:
           doc (xml): create_service request, choice customer|payer|subscriber

        Returns:
           xml: create_service_response with result true,
                SOAP fault when service not created            

        Example:

        .. code-block:: xml

           <create_service>
             <service>615</service>
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
             <status>active</active>
             <params>
               <entry>
                 <key>121</key>
                 <value>12345</value>
               </entry>
             </params>
           </create_service>   

           <create_service_response>
             <result>true</result>
           </create_service_response>                    

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'create_service', tostring(doc)),
                      self._mh.fromhere())

        service = doc.find('service').text if doc.find('service') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None
        status = doc.find('status').text if doc.find('status') else None

        params = {}
        if (doc.find('params') != None):
            for param in doc.findall('params/entry'):
                params[param.find('key').text] = param.find('value').text

        db = self._get_db()
        res = db.create_service(
            service, customer, payer, subscriber, status, params)
        db.disconnect()

        if (id != None):
            elem = Element('result')
            elem.text = 'true'
            return self._response('create_service', elem)
        else:
            return self._fault('Service not created')

    def change_service(self, doc):
        """Method handles change_service request         

        Args:
           doc (xml): change_service request, choice customer|payer|subscriber 

        Returns:
           xml: change_service_response with result true,
                SOAP fault when service not changed    

        Example:

        .. code-block:: xml

           <change_service>
             <service>615</service>
             <customer>1</customer>
             <payer>1</payer>
             <subscriber>1</subscriber>
             <status>active</active>
             <params>
               <entry>
                 <key>121</key>
                 <value>12345</value>
               </entry>
             </params>
           </change_service>  

           <change_service_response>
             <result>true</result>
           </change_service_response>                                     

        """

        self._mh.dmsg('htk_on_debug_info', self._mh._trn.msg('te_soap_request', 'change_service', tostring(doc)),
                      self._mh.fromhere())

        service = doc.find('service').text if doc.find('service') else None
        customer = doc.find('customer').text if doc.find('customer') else None
        payer = doc.find('payer').text if doc.find('payer') else None
        subscriber = doc.find('subscriber').text if doc.find(
            'subscriber') else None
        status = doc.find('status').text if doc.find('status') else None

        params = {}
        if (doc.find('params') != None):
            for param in doc.findall('params/entry'):
                params[param.find('key').text] = param.find('value').text

        db = self._get_db()
        res = db.change_service(
            service, customer, payer, subscriber, status, params)
        db.disconnect()

        if (id != None):
            elem = Element('result')
            elem.text = 'true'
            return self._response('change_service', elem)
        else:
            return self._fault('Service not changed')



# -*- coding: utf-8 -*-

"""This code is a part of Hydra Toolkit

.. module:: hydratk.extensions.testenv.translation.en.help
   :platform: Unix
   :synopsis: English language translation for TestEnv extension help generator
.. moduleauthor:: Petr Rašek <bowman@hydratk.org>

"""
language = {
    'name': 'English',
    'ISO-639-1': 'en'
}

''' TestEnv Commands '''
help_cmd = {
    'te-install': 'install testing environment database',
    'te-run': 'start testing environment',

    # standalone with option profile testenv
    'install': 'install testing environment database',
    'run': 'start testing environment'
}

''' TestEnv Options '''
help_opt = {
}
